package com.example.profile;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.TextView;

import com.demo.userprofile.R;
import com.xiaomi.opensdk.pdc.OpenPdcSyncServerAdapter;
import com.xiaomi.opensdk.pdc.OpenProfileManager;
import com.xiaomi.opensdk.pdc.UserProfile;
import com.xiaomi.opensdk.pdc.UserProfile.Gender;
import com.xiaomi.opensdk.pdc.exception.AuthenticationException;
import com.xiaomi.opensdk.pdc.exception.RetriableException;
import com.xiaomi.opensdk.pdc.exception.UnretriableException;

/**
 * Demo里AccessToken和ClientID写死，实际使用时ClientID为开发者网站上注册应用的AppID，
 * AccessToken需通过OAuth SDK获得用户授权后获得。
 * 因为Profile目前没有Clear接口，故Demo第一次运行时调用GetProfile就有值，望注意。
 */
public class MainActivity extends Activity {

    private static final String ACCESS_TOKEN =
            "V2_yVt8cILMTaCl7zk0xTiPCDhTQa1TZgA1NHC3fFBI-kEHYZqvoJ78k2PcVsJW2maCR3h643E0krOsjEhwNM3fmFI_iCBRGMP43LS-d8aRVIHz4kHlmgWSCo0RbkYQsqqqZOo4Jx5MKpY8-U5fEqj1OA";
    private static final String CLIENT_ID = "2882303761517338017";
    private OpenProfileManager mProfileManager;
    private Handler mMainThreadHanler;
    private Handler mSubThreadHanler;
    private HandlerThread mHandlerThread;
    private int mRetryTime;

    private EditText mBirthday;
    private RadioButton mMale;
    private RadioButton mFemale;
    private Button mSetButton;
    private Button mGetButton;
    private TextView mResult;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.activity_main);
        prepare();
        initView();
        initHandler();
    }

    private void prepare() {
        mProfileManager = new OpenProfileManager(new OpenPdcSyncServerAdapter.DefaultEnvironment(this),
                CLIENT_ID, ACCESS_TOKEN);
    }

    private void initView() {
        mBirthday = (EditText) findViewById(R.id.birthday);
        mMale = (RadioButton) findViewById(R.id.male);
        mFemale = (RadioButton) findViewById(R.id.female);

        mSetButton = (Button) findViewById(R.id.setProfile);
        mGetButton = (Button) findViewById(R.id.getProfile);

        mSetButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mRetryTime = 3;
                mSubThreadHanler.post(new Runnable() {
                    @Override
                    public void run() {
                        handleSetProfile();
                    }
                });
            }
        });

        mGetButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mRetryTime = 3;
                mSubThreadHanler.post(new Runnable() {
                    @Override
                    public void run() {
                        handleGetProfile();
                    }
                });
            }
        });

        mResult = (TextView) findViewById(R.id.content);
    }

    private void initHandler() {
        mMainThreadHanler = new Handler() {
            public void handleMessage(Message msg) {
                if (msg.obj != null && msg.obj instanceof String) {
                    String text = (String) msg.obj;
                    mResult.setText(text);
                }
            };
        };
        mHandlerThread = new HandlerThread("WorkingThread");
        mHandlerThread.start();
        mSubThreadHanler = new Handler(mHandlerThread.getLooper());
    }

    private void handleSetProfile() {
        UserProfile upload = new UserProfile();
        try {
            long b = Long.parseLong(mBirthday.getText().toString());
            upload.birthday = b;
        } catch (NumberFormatException e) {
        }
        if (mMale.isChecked()) {
            upload.gender = Gender.MALE;
        }
        if (mFemale.isChecked()) {
            upload.gender = Gender.FEMALE;
        }
        try {
            setText("start uploading");
            mProfileManager.setProfile(upload);
            setText("set sucess\n" + upload);
        } catch (UnretriableException e) {
            setText(e.toString());
        } catch (RetriableException e) {
            if (mRetryTime-- > 0) {
                try {
                    setText("Need Retry(" + (3 - mRetryTime) + "): After " + e.getRetryTime() / 1000 + " Seconds");
                    Thread.sleep(e.getRetryTime());
                    handleSetProfile();
                } catch (InterruptedException e1) {
                    setText(e1.toString());
                }
            } else {
                setText("error:retry more than 3 times");
            }
        } catch (AuthenticationException e) {
            // TODO 重获用户授权
            setText(e.toString());
        }
    }

    private void handleGetProfile() {
        UserProfile download;
        try {
            setText("start downloading");
            download = mProfileManager.getUserProfile();
            setText("get sucess\n" + download);
        } catch (UnretriableException e) {
            setText(e.toString());
        } catch (RetriableException e) {
            if (mRetryTime-- > 0) {
                try {
                    setText("Need Retry(" + (3 - mRetryTime) + "): After " + e.getRetryTime() / 1000 + " Seconds");
                    Thread.sleep(e.getRetryTime());
                    handleGetProfile();
                } catch (InterruptedException e1) {
                    setText(e1.toString());
                }
            } else {
                setText("error:retry more than 3 times");
            }
        } catch (AuthenticationException e) {
            // TODO 重获用户授权
            setText(e.toString());
        }
    }

    private void setText(String text) {
        Log.d("MiCloudOpenProfileDemo", text);
        Message msg = new Message();
        msg.obj = text;
        mMainThreadHanler.sendMessage(msg);
    }

}
